/**
 * Copyright (c) 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.api.command;

import java.util.Map;

import jline.console.completer.Completer;

import org.apache.commons.cli.Options;
import org.ow2.petals.cli.api.command.exception.CommandException;
import org.ow2.petals.cli.api.command.exception.CommandInvalidException;
import org.ow2.petals.cli.api.command.exception.ConnectionRequiredException;
import org.ow2.petals.cli.api.shell.Shell;

/**
 * The interface of a command executed by Petals CLI
 * 
 * @author Christophe DENEUX - Linagora
 */
public interface Command {
    
    /**
     * @return Command options
     */
    public Options getOptions();

    /**
     * After each call parsing, it is needed to reset {@link Options} of the
     * command.
     */
    public void resetOptions();

    /**
     * @return Command options description used by the command 'help'
     */
    public String getOptionsDescription();

    /**
     * @return the command name. Not <code>null</code>.
     */
    public String getName();

    /**
     * Get the usage message for this command
     * 
     * @return The command usage, a string describing command's arguments
     */
    public String getUsage();

    /**
     * Get the description of this command
     * 
     * @return A string describing the purpose and use of this command
     */
    public String getDescription();

    /**
     * Get the completer for the unique argument (Only one argument is allowed
     * for a command)
     * 
     * @return the completer for the unique argument or null if there is no
     *         completer
     */
    public Completer getDefaultCompleter();

    /**
     * Get the completers for the options
     * 
     * @return a map with short option as key and corresponding completer as
     *         value (If an option has no completer, the corresponding short
     *         option cannot be in the map)
     */
    public Map<String, Completer> getOptionCompleters();

    /**
     * Get the shell this command is bound to
     * 
     * @return a petals CLI shell
     */
    public Shell getShell();

    /**
     * Set the shell this command is bound to
     * 
     * @param shell
     *            The shell this command is bound to
     */
    public void setShell(final Shell shell);
    
    /**
     * Execute the command
     * 
     * @param args
     *            Arguments of the command
     * @throws CommandInvalidException
     *             When an error occurs about a command argument.
     * @throws CommandException
     *             An error occurs during the command parsing or command execution.
     * @throws ConnectionRequiredException
     *             The command requires an established connection
     */
    public void execute(final String[] args) throws CommandException;

    /**
     * @return <code>true</code> if the command requires a connection to a
     *         Petals ESB container, otherwise <code>false</code>.
     * @return
     */
    public boolean isConnectionRequired();

}
